@@ -4,19 +4,18 @@
 
 ;; Author: W. Borgert <debacle@debian.org>
 
-;; This program is free software; you can redistribute it and'or modify
-;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
-
-;; This program is distributed in the hope that it will be useful,
-;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;; GNU General Public License for more details.
-
-;; You should have received a copy of the GNU General Public License
-;; along with this program; if not, write to the Free Software
-;; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+; This program is free software: you can redistribute it and/or modify
+; it under the terms of the GNU General Public License as published by
+; the Free Software Foundation, either version 3 of the License, or
+; (at your option) any later version.
+;
+; This program is distributed in the hope that it will be useful,
+; but WITHOUT ANY WARRANTY; without even the implied warranty of
+; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+; GNU General Public License for more details.
+;
+; You should have received a copy of the GNU General Public License
+; along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 ;; @(#) $Id: plait.scm,v 1.14 2005/07/10 07:08:54 callum Exp $
 
@@ -264,15 +263,15 @@
 
 
 (define (button-pressed slot card-list)
-  (if (or (free? slot)
-	  (edge? slot)
-	  (= stock slot)
-	  (= plait slot))
-      #t
-      #f))
+  (and (or (free? slot)
+           (edge? slot)
+           (= stock slot)
+           (= plait slot))
+       (= (length card-list) 1)))
 
 (define (droppable? start-slot card-list end-slot)
-  (cond ((or (free? end-slot) (edge? end-slot)) (and
+  (cond ((= start-slot end-slot) #f)
+        ((or (free? end-slot) (edge? end-slot)) (and
 						 (= (length card-list) 1)
 						 (not (and (= start-slot plait)
 							   (free? end-slot)))
@@ -281,7 +280,8 @@
 	(else #f)))
 
 (define (button-released start-slot card-list end-slot)
-  (cond ((free? end-slot) (move-to-cell start-slot card-list end-slot))
+  (cond ((= start-slot end-slot) #f)
+        ((free? end-slot) (move-to-cell start-slot card-list end-slot))
 	((home? end-slot) (and
 			   (move-to-home card-list end-slot)
 			   (plait-to-edge start-slot)))
@@ -321,6 +321,7 @@
 ;;     used differently, depending on whether it's a home slot or not.
 (define (find-valid-move source slot-list)
   (and (not (null? slot-list))
+       (not (empty-slot? source))
        (let ((target (car slot-list)))
             (cond ((and (home? target)
                         (not (empty-slot? source))
